package org.openstreetmap.josm.plugins.lanetool.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JButton;
import javax.swing.JPanel;

import org.openstreetmap.josm.plugins.lanetool.data.LanesData;
import org.openstreetmap.josm.plugins.lanetool.view.style.DefaultStyle;
import org.openstreetmap.josm.plugins.lanetool.view.style.LanesStyle;

class LanesPanel extends JPanel {
	
	/* TODO: serialVersionUID */
	private static final long serialVersionUID = 1L;
	
	private LanesData laneData;
	private LanesStyle style = new DefaultStyle();
	
	private LaneGroupView laneGroupView;
	private JButton createRootLaneButton;
	
	private Component actionSourceComponent = this;
	
	public LanesPanel(LanesData laneData) {
		super();
		
		this.addMouseListener(new MouseListener());
		
		this.laneData = laneData;
		
		createRootLaneButton = new JButton("create lane group");
		createRootLaneButton.addActionListener(new CreateRootLaneListener());
		this.add(createRootLaneButton, BorderLayout.CENTER);		
		createRootLaneButton.setVisible(false);
	}
	
	@Override
	protected void paintComponent(Graphics g) {
			
		if (laneData.getLaneGroup() != null) {
			
			this.setBackground(Color.BLACK);
		
			createRootLaneButton.setVisible(false);
			
			super.paintComponent(g);
			
			g.drawImage(style.getBackgroundImage(), 0, 0, null);
			
			laneGroupView = new LaneGroupView(laneData.getLaneGroup(), style, 0, this.getHeight());
			laneGroupView.setXOffset(this.getWidth()/2);
			laneGroupView.paint(g);
			
		} else if (laneData.canCreateLaneGroup()) {
			
			this.setBackground(Color.BLACK);
			
			createRootLaneButton.setEnabled(true);
			createRootLaneButton.setVisible(true);
			
			super.paintComponent(g);
			
		} else {
			
			this.setBackground(Color.DARK_GRAY);
						
			createRootLaneButton.setEnabled(false);
			createRootLaneButton.setVisible(true);
			
			super.paintComponent(g);
			
		}
			
	}
		
	private class MouseListener extends MouseAdapter {
		
		@Override
		public void mouseClicked(MouseEvent e) {
			
			if (laneGroupView != null) {
				switch (e.getButton()) {
				case MouseEvent.BUTTON1: laneGroupView.handleLeftClick(actionSourceComponent, e.getX(), e.getY(), e.isShiftDown(), e.isControlDown()); break;
				case MouseEvent.BUTTON2:
				case MouseEvent.BUTTON3: laneGroupView.handleRightClick(actionSourceComponent, e.getX(), e.getY(), e.isShiftDown(), e.isControlDown()); break;
				}
			}
			
		}
	}
	
	private class CreateRootLaneListener implements ActionListener {

		public void actionPerformed(ActionEvent arg0) {
			if (laneData.canCreateLaneGroup()) {
				laneData.createLaneGroup();
			}
		}
		
	}
	
}
